#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for cluster environment setup for the given Amazon EC2 reservation
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_e, sh_command, Timer
from balloon.common import ssh

from balloon import amazon
from balloon.amazon import ec2


#--------------------------------------------------------------------------------------
def main() :
    #----------------------- Defining utility command-line interface -------------------------    
    import openmpi_config_options

    an_usage_description = "%prog"
    an_usage_description += ec2.use.options.usage_description()
    an_usage_description += openmpi_config_options.usage_description()

    from balloon import VERSION
    a_version = "%s" % VERSION

    from optparse import IndentedHelpFormatter
    a_help_formatter = IndentedHelpFormatter( width = 127 )

    from optparse import OptionParser
    an_option_parser = OptionParser( usage = an_usage_description, version = a_version, formatter = a_help_formatter )


    #----------------------- Definition of the command line arguments ------------------------
    openmpi_config_options.add( an_option_parser )

    ec2.use.options.add( an_option_parser )

    amazon.security_options.add( an_option_parser )
    
    common.options.add( an_option_parser )
  
 
    #------------------ Extracting and verifying command-line arguments ----------------------
    an_options, an_args = an_option_parser.parse_args()

    common.options.extract( an_option_parser )

    AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.security_options.extract( an_option_parser )

    a_reservation_id = ec2.use.options.extract( an_option_parser )

    a_hostfile = openmpi_config_options.extract( an_option_parser )
    
    from balloon.preferences import get
    an_image_location = get( 'amazon.image_location' )
    a_host_port = int( get( 'amazon.ec2.host_port' ) )
    a_login_name = get( 'amazon.ec2.ami.use.login_name' )


    print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
    import sys; an_engine = sys.argv[ 0 ]
    
    a_call = "%s %s %s" % ( an_engine, 
                            openmpi_config_options.compose( a_hostfile ),
                            ec2.use.options.compose( a_reservation_id ) )
    print_d( a_call + '\n' )


    print_d( "\n----------------------- Running actual functionality ----------------------\n" )
    a_spent_time = Timer()
    
    an_ec2_conn = ec2.common.region_connect( an_image_location, AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )

    try:
        a_reservation = ec2.use.get_reservation( an_ec2_conn, a_reservation_id )
        print_d( '< %r > : %s\n' % ( a_reservation, a_reservation.instances ) )
    except Exception, exc:
        print_e( '%s\n' % exc )
        pass

    a_security_group = ec2.use.get_security_group( an_ec2_conn, a_reservation )
    print_d( "< %r > : %s\n" % ( a_security_group, a_security_group.rules ) )


    print_d( "\n-------------------- Providing seamless ssh connection --------------------\n" )
    an_instance2ssh = {}
    for an_instance in a_reservation.instances :
        a_password = None
        an_identity_file = ec2.run.get_identity_filepath( an_instance.key_name )
        a_host_name = an_instance.public_dns_name
        ssh.options.echo( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )
        
        a_ssh_client = ssh.connect( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )
        an_instance2ssh[ an_instance ] = a_ssh_client

        import os.path; an_upload_name = os.path.basename( an_identity_file )

        a_sftp_client = a_ssh_client.open_sftp()
        a_sftp_client.put( an_identity_file, an_upload_name )

        a_target_name = '${HOME}/.ssh/id_rsa'
        ssh.command( a_ssh_client, 'mv -f %s %s' % ( an_upload_name, a_target_name ) )
        ssh.command( a_ssh_client, 'chmod 600 %s' % ( a_target_name ) )
        ssh.command( a_ssh_client, """sudo sh -c 'echo "    StrictHostKeyChecking no" >> /etc/ssh/ssh_config'""" )
        
        try:
            a_security_group.authorize( 'tcp', 1, 65535, '%s/0' % an_instance.private_ip_address ) # mpi cluster ports
        except :
            pass
        pass

    print_d( "\n--- Listing all the cluster nodes into special '.openmpi_hostfile' file ---\n" )
    # The suggested cluster conffiguration is symmetric,
    # which means that it does not matter who will be the master node.
    # (if user need to do something special, he can do it from his own machine)
    for a_master_node in a_reservation.instances :
        a_host_name = a_master_node.public_dns_name
        
        a_ssh_client = an_instance2ssh[ a_master_node ]
        
        a_number_cpu = ssh.command( a_ssh_client, """python -c 'import os; print os.sysconf( "SC_NPROCESSORS_ONLN" )'""" )[ 0 ][ : -1 ]
        a_number_cpu = int( a_number_cpu )

        ssh.command( a_ssh_client, 'echo %s slots=%d > %s' % ( a_master_node.private_ip_address, a_number_cpu, a_hostfile ) )
        for an_instance in a_reservation.instances :
            if an_instance == a_master_node :
                continue

            ssh.command( a_ssh_client, 'echo %s slots=%d >> %s' % ( an_instance.private_ip_address, a_number_cpu, a_hostfile ) )
            pass
        
        pass
    
    [ a_ssh_client.close() for a_ssh_client in an_instance2ssh.values() ]
    
    print_d( "a_spent_time = %s, sec\n" % a_spent_time )
    
    
    print_d( "\n------------------ Printing succussive pipeline arguments -----------------\n" )
    ec2.use.options.track( a_reservation_id )
    

    print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
    print_d( a_call + '\n' )


    print_d( "\n-------------------------------------- OK ---------------------------------\n" )
    pass


#------------------------------------------------------------------------------------------
if __name__ == '__main__' :
    main()
    pass


#------------------------------------------------------------------------------------------



#!/usr/bin/env python

#------------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#------------------------------------------------------------------------------------------
"""
Removes whole appointed cloud study or just given files from this study
"""

#------------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_i, print_e, sh_command
from balloon.common import Timer, WorkerPool, compute_md5

import balloon.amazon as amazon
from balloon.amazon.s3 import TRootObject, TStudyObject, TFileObject, TSeedObject

import sys, os, os.path, uuid, hashlib


#--------------------------------------------------------------------------------------
def main() :
    #----------------------- Defining utility command-line interface -------------------------    
    an_usage_description = "%prog"

    from rm_options import usage_description as usage_description_options
    an_usage_description += usage_description_options()
    
    from balloon import VERSION
    a_version = "%s" % VERSION

    from optparse import IndentedHelpFormatter
    a_help_formatter = IndentedHelpFormatter( width = 127 )

    from optparse import OptionParser
    an_option_parser = OptionParser( usage = an_usage_description, version = a_version, formatter = a_help_formatter )


    #----------------------- Definition of the command line arguments ------------------------
    from rm_options import add as add_options
    add_options( an_option_parser )

    amazon.security_options.add( an_option_parser )
    
    common.concurrency_options.add( an_option_parser )
    
    common.communication_options.add( an_option_parser )
    
    common.options.add( an_option_parser )


    #------------------ Extracting and verifying command-line arguments ----------------------
    an_options, an_args = an_option_parser.parse_args()

    common.options.extract( an_option_parser )

    common.communication_options.extract( an_option_parser )
    
    a_number_threads = common.concurrency_options.extract( an_option_parser )
    
    AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.security_options.extract( an_option_parser )

    from rm_options import extract as extract_options
    a_study_name, a_located_files = extract_options( an_option_parser )


    print_i( "--------------------------- Looking for study object ----------------------------\n" )
    a_root_object = TRootObject.get( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
    print_d( "a_root_object = %s\n" % a_root_object )

    a_study_object = TStudyObject.get( a_root_object, a_study_name )
    print_d( "a_study_object = %s\n" % a_study_object )


    print_i( "-------------------------- Removing study files -----------------------------\n" )
    a_worker_pool = WorkerPool( len( a_located_files ) )

    a_deleter = lambda the_object, the_number_threads, the_printing_depth : \
        the_object.delete( the_number_threads, the_printing_depth )

    for a_located_file in a_located_files :
        print_d( "a_located_file = %s\n" % a_located_file )
    
        a_file_object = TFileObject.get( a_study_object, a_located_file )
        print_d( "a_file_object = %s\n" % a_file_object, 1 )
        
        a_worker_pool.charge( a_deleter, ( a_file_object, a_number_threads, 2 ) )
        pass

    a_worker_pool.shutdown()
    a_worker_pool.join()


    print_i( "------------------- Printing succussive pipeline arguments ----------------------\n" )
    print a_study_name


    print_i( "-------------------------------------- OK ---------------------------------------\n" )
    pass


#------------------------------------------------------------------------------------------
if __name__ == '__main__' :
    main()
    pass


#------------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script upload a file from web and publish it as 'study file' in S3
"""


#--------------------------------------------------------------------------------------
def usage_description() :
    return " --remote-location='/mnt'"


#--------------------------------------------------------------------------------------
from balloon.preferences import OptionGroup, PersistentOption, TransientOption
a_container = OptionGroup( 'amazon.apps.deploy_credentials' )

a_container.add_option( TransientOption( "--aws-user-id",
                                         metavar = "< AWS User ID >",
                                         action = "store",
                                         dest = "aws_user_id",
                                         help = "(${AWS_USER_ID}, by default)",
                                         default = None ) )

a_container.add_option( TransientOption( "--ec2-private-key",
                                         metavar = "< EC2 Private Key >",
                                         action = "store",
                                         dest = "ec2_private_key",
                                         help = "(${EC2_PRIVATE_KEY}, by default)",
                                         default = None ) )

a_container.add_option( TransientOption( "--ec2-cert",
                                         metavar = "< EC2 Certificate >",
                                         action = "store",
                                         dest = "ec2_cert",
                                         help = "(${EC2_CERT}, by default)",
                                         default = None ) )

a_container.add_option( PersistentOption( "--remote-location",
                                          metavar = "< destination of the credendtials environemnt files >",
                                          action = "store",
                                          dest = "remote_location",
                                          help = "(\"%default\", by default)",
                                          default = '/tmp/.aws_credentialsrc' ) )


#--------------------------------------------------------------------------------------
def extract( the_option_parser ) :
    from balloon.preferences import resource_filename
    an_options, an_args = the_option_parser.parse_args()

    AWS_USER_ID = an_options.aws_user_id
    if AWS_USER_ID == None :
        import os; AWS_USER_ID = os.getenv( "AWS_USER_ID" )
        pass

    EC2_PRIVATE_KEY = an_options.ec2_private_key
    if EC2_PRIVATE_KEY == None :
        import os; EC2_PRIVATE_KEY = os.getenv( "EC2_PRIVATE_KEY" )
        pass

    if EC2_PRIVATE_KEY != None :
        EC2_PRIVATE_KEY = os.path.abspath( EC2_PRIVATE_KEY )
        if not os.path.isfile( EC2_PRIVATE_KEY ) :
            the_option_parser.error( "--ec2-private-key='%s' must be a file" % EC2_PRIVATE_KEY )
            pass
        pass

    EC2_CERT = an_options.ec2_cert
    if EC2_CERT == None :
        import os; EC2_CERT = os.getenv( "EC2_CERT" )
        pass

    if EC2_PRIVATE_KEY != None :
        EC2_CERT = os.path.abspath( EC2_CERT )
        if not os.path.isfile( EC2_CERT ) :
            the_option_parser.error( "--ec2-cert='%s' must be a file" % EC2_CERT )
            pass
        pass

    a_remote_location = os.path.abspath( an_options.remote_location )

    return AWS_USER_ID, EC2_PRIVATE_KEY, EC2_CERT, a_remote_location


#--------------------------------------------------------------------------------------
def compose( AWS_USER_ID, EC2_PRIVATE_KEY, EC2_CERT, the_remote_location ) :
    return "--remote-location='%s'" % ( the_remote_location )


#--------------------------------------------------------------------------------------
from balloon.preferences import template_add
add = lambda the_option_parser : template_add( the_option_parser, a_container )


#------------------------------------------------------------------------------------------
from balloon.preferences import template_dump
dump = lambda the_identation_level, the_output : template_dump( the_identation_level, a_container, the_output )


#------------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for cluster environment setup for the given Amazon EC2 reservation
"""


#--------------------------------------------------------------------------------------
def usage_description() :
    return " --case-dir='./damBreak' --output-suffix='.out' --before-hook='balloon-foam2vtk-before' --time-hook='balloon-foam2vtk' --after-hook='balloon-foam2vtk-after'"


#--------------------------------------------------------------------------------------
from balloon.preferences import OptionGroup, PersistentOption, TransientOption
a_container = OptionGroup( 'amazon.apps.solver_run' )

a_container.add_option( TransientOption( "--case-dir",
                                         metavar = "< location of the source OpenFOAM case dir >",
                                         action = "store",
                                         dest = "case_dir",
                                         default = None ) )

a_container.add_option( PersistentOption( "--output-suffix",
                                          metavar = "< folder suffix for the output results >",
                                          action = "store",
                                          dest = "output_suffix",
                                          help = "('%default', by default)",
                                          default = '.out' ) )

a_container.add_option( PersistentOption( "--run-hook",
                                          metavar = "< case dir executable in to be run in the cluster >",
                                          action = "store",
                                          dest = "run_hook",
                                          help = "('%default', by default)",
                                          default = 'Allrun' ) )

a_container.add_option( PersistentOption( "--before-hook",
                                          metavar = "< executable to be run 'before' any results will be downloaded >",
                                          action = "store",
                                          dest = "before_hook",
                                          help = "('do nothing', by default)",
                                          default = None ) )

a_container.add_option( PersistentOption( "--time-hook",
                                          metavar = "< executable to be run on each timestamp download >",
                                          action = "store",
                                          dest = "time_hook",
                                          help = "('do nothing', by default)",
                                          default = None ) )

a_container.add_option( PersistentOption( "--after-hook",
                                          metavar = "< executable to be run 'after' all results will be downloaded >",
                                          action = "store",
                                          dest = "after_hook",
                                          help = "('do nothing', by default)",
                                          default = None ) )


#--------------------------------------------------------------------------------------
def extract( the_option_parser ) :
    an_options, an_args = the_option_parser.parse_args()

    if an_options.case_dir == None :
        the_option_parser.error( "Use '--case-dir' option to define folder containing solver case\n" )
        pass

    import os.path; a_case_dir = os.path.abspath( os.path.expanduser(an_options.case_dir ) )
    if not os.path.isdir( a_case_dir ) :
        the_option_parser.error( "--case-dir='%s' should be a folder\n" % a_case_dir )
        pass

    an_output_suffix = an_options.output_suffix

    a_run_hook = an_options.run_hook

    a_before_hook = an_options.before_hook
    a_time_hook = an_options.time_hook
    a_after_hook = an_options.after_hook

    return a_case_dir, an_output_suffix, a_run_hook, a_before_hook, a_time_hook, a_after_hook


#--------------------------------------------------------------------------------------
def compose( the_case_dir, the_output_suffix, the_run_hook, the_before_hook, the_time_hook, the_after_hook ) :
    a_compose = "--case-dir='%s' --output-suffix='%s' --run-hook='%s'" % ( the_case_dir, the_output_suffix, the_run_hook )

    if the_before_hook != None :
        a_compose += " --before-hook='%s'" % the_before_hook
        pass

    if the_time_hook != None :
        a_compose += " --time-hook='%s'" % the_time_hook
        pass

    if the_after_hook != None :
        a_compose += " --after-hook='%s'" % the_after_hook
        pass

    return a_compose


#--------------------------------------------------------------------------------------
from balloon.preferences import template_add
add = lambda the_option_parser : template_add( the_option_parser, a_container )


#------------------------------------------------------------------------------------------
from balloon.preferences import template_dump
dump = lambda the_identation_level, the_output : template_dump( the_identation_level, a_container, the_output )


#------------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for the task packaging and sending it for execution in a cloud
"""

#--------------------------------------------------------------------------------------
import sys, os, os.path

# To avoid using previoulsy cached contents for the distributed package
an_engine = sys.argv[ 0 ]
an_engine_dir = os.path.abspath( os.path.dirname( sys.argv[ 0 ] ) )
a_manifest_file = os.path.join( an_engine_dir, 'MANIFEST' )
if os.path.isfile( a_manifest_file ) :
    os.remove( a_manifest_file )
    pass

an_engine = os.path.basename( an_engine )

# To generate list of automatically distributed scripts
a_scripts = []
for a_file in os.listdir( an_engine_dir ) :
    if a_file == an_engine :
        continue

    if os.path.isfile( a_file ) : 
        if os.access( a_file, os.X_OK ) :
            a_scripts.append( a_file )
            pass
        pass
    pass


#-------------------------------------------------------------------------------------
import ez_setup
ez_setup.use_setuptools( '0.6c9' )

from setuptools import setup, find_packages
import balloon

os.chdir( an_engine_dir ) # Run from the proper folder

setup( name = balloon.NAME,
       description = 'Set of cloud computing automation utilities',
       long_description = 
       """These utilities provide seemless mode for:
        - accessing to cloud;
        - deploying user specified data and functionality to be run on this data;
        - launching of the cloud specified task;
        - storing of output data into cloud;
        - fectching of these output results from cloud;
        - other miscellaneous functions""",
       author = 'Alexey Petrov',
       author_email = 'alexey.petrov.nnov@gmail.com', 
       license = 'Apache License, Version 2.0',
       url = 'http://sourceforge.net/projects/balloon-foam',
       install_requires = [ 'boto', 'workerpool', 'paramiko', 'pexpect' ],
       platforms = [ 'linux' ],
       version = balloon.VERSION,
       classifiers = [ 'Development Status :: 3 - Alpha',
                       'Environment :: Console',
                       'Intended Audience :: Science/Research',
                       'License :: OSI Approved :: Apache Software License',
                       'Operating System :: POSIX',
                       'Programming Language :: Python',
                       'Topic :: Scientific/Engineering',
                       'Topic :: Utilities' ],
       packages = find_packages(),
       scripts = a_scripts,
       entry_points = { 'console_scripts': [
           'amazon_location_list.py = balloon.amazon.apps.location_list:main',
           'balloon-cluster-regions = balloon.amazon.apps.location_list:main',

           'amazon_reservation_run.py = balloon.amazon.apps.reservation_run:main',
           'balloon-reservation-run = balloon.amazon.apps.reservation_run:main',

           'amazon_reservation_list.py = balloon.amazon.apps.reservation_list:main',
           'balloon-cluster-ls = balloon.amazon.apps.reservation_list:main',

           'amazon_reservation_pickup.py = balloon.amazon.apps.reservation_pickup:main',

           'amazon_reservation_delete.py = balloon.amazon.apps.reservation_delete:main',
           'balloon-cluster-rm = balloon.amazon.apps.reservation_delete:main',

           'amazon_instance_extract.py = balloon.amazon.apps.instance_extract:main',
           'balloon-instance-extract = balloon.amazon.apps.instance_extract:main',

           'balloon_ssh.py = balloon.common.ssh.run:main',
           'balloon-ssh = balloon.common.ssh.run:main',

           'amazon_openmpi_config.py = balloon.amazon.apps.openmpi_config:main',
           'balloon-openmpi-config = balloon.amazon.apps.openmpi_config:main',

           'balloon-timestamps-upload = balloon.amazon.apps.timestamps_upload:main',

           'amazon_solver_run.py = balloon.amazon.apps.solver_run:main',
           'balloon-solver-run = balloon.amazon.apps.solver_run:main',

           'amazon_nfs_config.py = balloon.amazon.apps.nfs_config:main',
           'balloon-nfs-config = balloon.amazon.apps.nfs_config:main',

           'amazon_deploy_credentials.py = balloon.amazon.apps.deploy_credentials:main',
           'balloon-deploy-credentials = balloon.amazon.apps.deploy_credentials:main',

           'balloon_deploy.py = balloon.common.deploy:main',
           'balloon-deploy = balloon.common.deploy:main',

           'balloon-study-book = balloon.amazon.apps.study_book:main',
           'balloon-study-seal = balloon.amazon.apps.study_seal:main',

           'amazon_upload_start.py = balloon.amazon.apps.upload_start:main',
           'balloon-upload-start = balloon.amazon.apps.upload_start:main',

           'amazon_upload_resume.py = balloon.amazon.apps.upload_resume:main',
           'balloon-upload-resume = balloon.amazon.apps.upload_resume:main',

           'balloon-data-upload = balloon.amazon.apps.data_upload:main',

           'amazon_download.py = balloon.amazon.apps.download:main',
           'balloon-download = balloon.amazon.apps.download:main',

           'amazon_ls.py = balloon.amazon.apps.ls:main',
           'balloon-ls = balloon.amazon.apps.ls:main',

           'amazon_rm_study.py = balloon.amazon.apps.rm_study:main',
           'balloon-rm-study = balloon.amazon.apps.rm_study:main',

           'amazon_rm.py = balloon.amazon.apps.rm:main',
           'balloon-rm = balloon.amazon.apps.rm:main'
           ] },
       zip_safe = True )


#--------------------------------------------------------------------------------------





#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
import options
import communication_options
import concurrency_options
import ssh


#--------------------------------------------------------------------------------------
import os, os.path, sys, hashlib
from subprocess import *


#---------------------------------------------------------------------------
import threading; PRINTING_LOCK = threading.Lock()
PRINTING_DEPTH = 0

def get_preffix( the_preffix, the_printing_depth ) :
    "Calaculate a preffix identation according to the current printing stack depth"
    a_preffix = ""
    for id in range( the_printing_depth ) :
        a_preffix += the_preffix
        pass

    return a_preffix


#---------------------------------------------------------------------------
def print_d( the_message, the_printing_depth = 0 ) :
    "Optional printing of debug messages"
    from options import is_debug

    if is_debug() : 
        PRINTING_LOCK.acquire()

        sys.stderr.write( get_preffix( "    ", the_printing_depth ) + the_message )

        PRINTING_LOCK.release()
        pass
    
    pass


#---------------------------------------------------------------------------
def print_i( the_message, the_printing_depth = 0 ) :
    "Optional printing of debug messages"
    print_d( "\n" )

    print_d( the_message, the_printing_depth )

    pass


#---------------------------------------------------------------------------
def print_list( the_list, the_printing_depth = 0 ) :
    "Optional printing of debug messages"
    from options import is_debug

    if not is_debug() : 
        return

    for an_item in the_list :
        print_d( "%s\n" % ( an_item ), the_printing_depth ) 
        pass
    pass


#---------------------------------------------------------------------------
def print_traceback( the_printing_depth = 0 ) :
    "Optional printing of debug messages"
    import cStringIO, traceback

    a_stream = cStringIO.StringIO()
    traceback.print_exc( file = a_stream )
    
    print_list( a_stream.getvalue().split( "\n" ), the_printing_depth )
    pass


#---------------------------------------------------------------------------
def print_e( the_message, the_exit = True ) :
    "Printing of error message and quit"
    PRINTING_LOCK.acquire()

    sys.stderr.write( "\n" )
    sys.stderr.write( "---------------------------------- ERROR ----------------------------------\n" )

    sys.stderr.write( the_message )
    
    sys.stderr.write( "----------------------------------- KO ------------------------------------\n" )

    PRINTING_LOCK.release()

    if the_exit == True :
        os._exit( os.EX_USAGE )
        pass

    pass


#---------------------------------------------------------------------------
def sh_command( the_command, the_printing_depth = 0 ) :
    "Execution of shell command"
    print_d( "(%s)\n" % the_command, the_printing_depth )
    
    a_pipe = Popen( the_command, stdout = PIPE, stderr = PIPE, shell = True )

    a_return_code = os.waitpid( a_pipe.pid, 0 )[ 1 ]

    a_stderr_lines = a_pipe.stderr.readlines()
    for a_line in a_stderr_lines : print_d( "\t%s" % a_line )

    a_stdout_lines = a_pipe.stdout.readlines()
    for a_line in a_stdout_lines : print_d( "\t%s" % a_line )

    if a_return_code != 0 :
        raise OSError( "(%s)\n" % the_command )
        pass
    
    return a_stdout_lines


#--------------------------------------------------------------------------------------
def arg_list_separator():
    import os
    if os.getenv( "__BALLOON_ARG_LIST_SEPARATOR__" ) != None:
       return os.getenv( "__BALLOON_ARG_LIST_SEPARATOR__" )

    return '|'


#---------------------------------------------------------------------------
def print_args( the_args, the_separator ) :
    "Prints command-line args through given  separator"
    a_result = ''
    an_args_length = len( the_args )
    for an_id in range( an_args_length ) :
        a_result += the_args[ an_id ]
        if an_id == an_args_length - 1 :
            break

        a_result += the_separator
        pass

    return a_result


#--------------------------------------------------------------------------------------
class Timer :
    def __init__( self ) :
        import time
        self.initial = time.time()
        pass

    def __str__( self ) :
        import time
        return str( "%f" % ( time.time() - self.initial ))

    pass


#------------------------------------------------------------------------------------------
import workerpool
from workerpool.pools import default_worker_factory

class WorkerPool( workerpool.WorkerPool ) :
    "Run all the registered tasks in parallel"
    def __init__( self, size = 1, maxjobs = 0, worker_factory = default_worker_factory ) :
        workerpool.WorkerPool.__init__( self, size, maxjobs, worker_factory )

        from workerpool.pools import Queue
        self.results = Queue()
        pass

    def charge( self, the_function, *the_args ) :
        "Perform a map operation distributed among the workers. Will block until done."
        from workerpool import SimpleJob
        self.put( SimpleJob( self.results, the_function, *the_args ) )

        pass

    def is_all_right( self ) :
        self.join()

        a_result = True
        for i in range( self.results.qsize() ):
            a_result &= self.results.get()
            self.results.task_done()
            pass

        return a_result

    pass


#--------------------------------------------------------------------------------------
class TaggedWorkerPool( WorkerPool ) :
    "Run all the registered tasks in parallel"
    def __init__( self, size = 1, maxjobs = 0, worker_factory = default_worker_factory ) :
        WorkerPool.__init__( self, size, maxjobs, worker_factory )

        self._unique_tag_names = set()
        pass

    def charge( self, the_tag_name, the_function, *the_args ) :
        "Perform a map operation distributed among the workers. Will block until done."

        if the_tag_name != None :
            if the_tag_name in self._unique_tag_names :
                return 

            self._unique_tag_names.add( the_tag_name )
            pass

        WorkerPool.charge( self, the_function, *the_args )
        pass

    pass


#--------------------------------------------------------------------------------------
def compute_md5( the_file_pointer ) :
    from boto.s3.key import Key

    return Key().compute_md5( the_file_pointer )


#--------------------------------------------------------------------------------------





#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
Contains the package dedicated preferences
"""


#--------------------------------------------------------------------------------------
def usage_description() :
    return " [--production]"


#--------------------------------------------------------------------------------------
from balloon.preferences import OptionGroup, PersistentOption, TransientOption
a_container = OptionGroup( 'common.install' )

a_container.add_option( PersistentOption( "--production",
                                          metavar = "< deploy official production version >",
                                          action = "store_true",
                                          dest = "production",
                                          help = "(%default, by default)",
                                          default = True ) )

a_container.add_option( PersistentOption( "--url",
                                          metavar = "< package name or precise location to download from >",
                                          action = "store",
                                          dest = "url",
                                          help = "('%default', by default)",
                                          default = 'balloon' ) )


#--------------------------------------------------------------------------------------
def extract( the_option_parser ) :
    an_options, an_args = the_option_parser.parse_args()

    a_production = an_options.production

    an_url = an_options.url

    return a_production, an_url


#--------------------------------------------------------------------------------------
def compose( the_production, the_url ) :
    if the_production == True :
        return "--production --url='%s'" % the_url

    return ""


#--------------------------------------------------------------------------------------
from balloon.preferences import template_add
add = lambda the_option_parser : template_add( the_option_parser, a_container )


#------------------------------------------------------------------------------------------
from balloon.preferences import template_dump
dump = lambda the_identation_level, the_output : template_dump( the_identation_level, a_container, the_output )


#------------------------------------------------------------------------------------------





#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
Contains the package dedicated preferences
"""


#--------------------------------------------------------------------------------------
def usage_description() :
    return " --image-location='us-east-1'"


#--------------------------------------------------------------------------------------
from balloon.preferences import OptionGroup, PersistentOption, TransientOption
a_container = OptionGroup( 'amazon' )

a_container.add_option( PersistentOption( "--image-location",
                                          metavar = "< location of the cluster nodes : 'eu-west-1', 'us-east-1', 'us-west-1' or 'ap-southeast-1' >",
                                          choices = [ 'eu-west-1', 'us-east-1', 'us-west-1', 'ap-southeast-1' ],
                                          action = "store",
                                          dest = "image_location",
                                          help = "('%default', by default)",
                                          default = "us-east-1" ) )


#--------------------------------------------------------------------------------------
def extract( the_option_parser ) :
    an_options, an_args = the_option_parser.parse_args()

    an_image_location = an_options.image_location

    return an_image_location


#--------------------------------------------------------------------------------------
def compose( the_image_location ) :
    return "--image-location='%s'" % ( the_image_location )


#--------------------------------------------------------------------------------------
from balloon.preferences import template_add
add = lambda the_option_parser : template_add( the_option_parser, a_container )


#------------------------------------------------------------------------------------------
def dump( the_identation_level, the_output ) :
    from balloon.preferences import dump_begin
    dump_begin( the_identation_level, a_container, the_output )

    from security_options import dump as security_dump
    security_dump( the_identation_level + 1, the_output )
    
    from ec2.options import dump as ec2_dump
    ec2_dump( the_identation_level + 1, the_output )
    
    from s3_options import dump as s3_dump
    s3_dump( the_identation_level + 1, the_output )
    
    from apps.options import dump as apps_dump
    apps_dump( the_identation_level + 1, the_output )
    
    from balloon.preferences import dump_end
    dump_end( the_identation_level, a_container, the_output )
    pass


#------------------------------------------------------------------------------------------




#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
Contains the package dedicated preferences
"""


#--------------------------------------------------------------------------------------
def usage_description() :
    return " --instance-type='m1.large' --image-id='ami-1cdb2c75'"


#--------------------------------------------------------------------------------------
from balloon.preferences import OptionGroup, PersistentOption, TransientOption
a_container = OptionGroup( 'amazon.ec2.ami.run' )

a_container.add_option( PersistentOption( "--instance-type",
                                          metavar = "< EC2 instance type : 'c1.xlarge' or 'm1.large', for example >",
                                          choices = [ 'm1.large', 'c1.xlarge', 'm1.xlarge', ' m2.xlarge', 'm2.2xlarge', 'm2.4xlarge', 'c1.medium' ],
                                          action = "store",
                                          dest = "instance_type",
                                          help = "('%default', by default)",
                                          default = "c1.xlarge" ) )

a_container.add_option( PersistentOption( "--image-id",
                                          metavar = "< Amazon EC2 AMI ID >",
                                          action = "store",
                                          dest = "image_id",
                                          help = "('%default', by default)",
                                          default = "ami-1cdb2c75" ) ) # ami-dec135b7, ami-0440b46d, ami-3a877053


#--------------------------------------------------------------------------------------
def extract( the_option_parser ) :
    an_options, an_args = the_option_parser.parse_args()

    an_instance_type = an_options.instance_type
    an_image_id = an_options.image_id

    return an_instance_type, an_image_id


#--------------------------------------------------------------------------------------
def compose( the_image_id, the_instance_type ) :
    return "--instance-type='%s' --image-id='%s'" % ( the_instance_type, the_image_id )


#--------------------------------------------------------------------------------------
from balloon.preferences import template_add
add = lambda the_option_parser : template_add( the_option_parser, a_container )


#------------------------------------------------------------------------------------------
from balloon.preferences import template_dump
dump = lambda the_identation_level, the_output : template_dump( the_identation_level, a_container, the_output )


#------------------------------------------------------------------------------------------




#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
import optparse
class OptionGroup( optparse.OptionContainer ) :
    def __init__( self, the_namespace, the_title = None, the_description = None ) :
        optparse.OptionContainer.__init__( self, optparse.Option, "error", the_description )
        self._create_option_mappings()

        self._namespace = the_namespace
        self._title = the_title
        pass

    def _create_option_list( self ) :
        self.option_list = []
        pass

    def get_namespace( self ) :
        return self._namespace
    
    def get_subnamespace( self ) :
        return self._namespace.split( '.' )[ -1 ]
    
    def get_title( self ) :
        return self._title
    
    def __len__( self ) :
        return len( self.option_list )
    
    pass


#--------------------------------------------------------------------------------------
import optparse
class PersistentOption( optparse.Option ) :
    def __init__( self, *opts, **attrs ) :
        optparse.Option.__init__( self, *opts, **attrs )
        pass

    pass


#--------------------------------------------------------------------------------------
import optparse
class TransientOption( optparse.Option ) :
    def __init__( self, *opts, **attrs ) :
        optparse.Option.__init__( self, *opts, **attrs )
        pass

    pass


#--------------------------------------------------------------------------------------
def resource_filename() :
    # If user explicitly appointed a specific resource file - use it
    import os; an_rcfilename = os.getenv( "BALLOON_RCFILE" )

    if an_rcfilename == None or an_rcfilename == '' : # If no user defined file, generate a proper one
        import balloon; an_rcfilename = "~/.%src-%s.py" % ( balloon.NAME, balloon.VERSION )
        pass

    return an_rcfilename


#--------------------------------------------------------------------------------------
IDENTATION_PREFFIX = "    "

def ident( the_identation_level ) :
    an_identation = ""
    for id in range( the_identation_level ) :
        an_identation += IDENTATION_PREFFIX
        pass
    
    return an_identation


#--------------------------------------------------------------------------------------
def dump_resume( the_identation_level, the_option_group, the_output ) :
    for an_option in the_option_group.option_list :
        if not isinstance( an_option, PersistentOption ) :
            continue
        
        a_dest = an_option.dest
        a_default = an_option.default
        a_metavar = an_option.metavar
        a_metavar = a_metavar.replace( '< ', '' )
        a_metavar = a_metavar.replace( ' >', '' )
        # a_metavar = a_metavar.title()
        a_metavar = a_metavar[ 0 ].upper() + a_metavar[ 1 : ]
        a_value_format = "%r"
        if isinstance( a_default, str ) :
            a_value_format = "'%s'"
            pass
        a_formatted_value = a_value_format % a_default
        the_output.write( ident( the_identation_level + 1 ) + "%s = %s # %s\n" %
                          ( a_dest, a_formatted_value, a_metavar ) )

        pass
    pass


#--------------------------------------------------------------------------------------
def dump_begin( the_identation_level, the_option_group, the_output ) :
    # if len( the_option_group ) == 0 :
    #     return
    
    a_section_comment = ''
    a_title = the_option_group.get_title()
    if a_title != None :
        a_section_comment = ' # %s' % a_title
        pass

    a_section_name = the_option_group.get_subnamespace();
    the_output.write( ident( the_identation_level ) + "class %s :%s\n" % ( a_section_name, a_section_comment ) )

    a_description = the_option_group.get_description()
    if a_description != None :
        the_output.write( ident( the_identation_level + 1 ) + "\"\"\" %s \"\"\"\n" % ( a_description ) )
        pass
    
    dump_resume( the_identation_level, the_option_group, the_output )

    # the_output.write( "\n" )
    pass
    

#--------------------------------------------------------------------------------------
def dump_end( the_identation_level, the_option_group, the_output ) :
    # if len( the_option_group ) == 0 :
    #     return

    the_output.write( ident( the_identation_level + 1 ) + "pass\n\n" )
    pass


#--------------------------------------------------------------------------------------
__PICKUP_CALLED__ = None

def pickup() :
    global __PICKUP_CALLED__
    if __PICKUP_CALLED__ != None:
        return
    
    from preferences import resource_filename
    an_rcfilename = resource_filename()

    import os.path;
    an_rcfilename = os.path.expanduser( an_rcfilename )
    an_rcfilename = os.path.abspath( an_rcfilename )
    if not os.path.isfile( an_rcfilename ) :
        # If resource file does not exist - write a predefined one
        an_rcfile = open( an_rcfilename, "w" )
        import balloon; an_rcfile.write( """
#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Automatically generated (%s-%s)
##


#--------------------------------------------------------------------------------------
REGION2PARAMS = { 'eu-west-1' : [ 'EU', 'ami-12ffca66' ],
                  'us-east-1' : [ '', 'ami-1cdb2c75' ], 
                  'us-west-1' : [ 'us-west-1', 'ami-24500061' ], 
                  'ap-southeast-1' : [ 'ap-southeast-1', 'ami-0c6d135e' ] }

class TRegionConfigHelper :
    def __init__( self, the_ec2_region ) :
        self._image_location = the_ec2_region
        self._image_id = REGION2PARAMS[ the_ec2_region ][ 1 ]
        self._s3location = REGION2PARAMS[ the_ec2_region ][ 0 ]
        pass

    def image_location( self ) :
        return self._image_location

    def image_id( self ) :
        return self._image_id
    
    def s3location( self ) :
        return self._s3location
    
    pass

a_region_config = TRegionConfigHelper( 'eu-west-1' )


#--------------------------------------------------------------------------------------
""" % ( balloon.NAME, balloon.VERSION ) )

        import cStringIO
        an_output = cStringIO.StringIO()

        from common.options import dump as common_dump;  common_dump( 0, an_output )

        from amazon.options import dump as amazon_dump;  amazon_dump( 0, an_output )
        
        an_rcfile.write( an_output.getvalue() )

        an_rcfile.write( '\n' )
        an_rcfile.write( '#--------------------------------------------------------------------------------------' )
        an_rcfile.write( '\n' )
    
        an_rcfile.close()

        import stat
        # os.chmod( an_rcfilename, stat.S_IRUSR ) # To protect user data
        pass

    __PICKUP_CALLED__ = True

    pass

    
#--------------------------------------------------------------------------------------
def get( the_namespace ) :
    a_value = None
    try:
        import pickup
        an_expression = 'a_value = pickup.%s' % ( the_namespace )

        exec an_expression

        if a_value != None and a_value != 'None' :
            return a_value
        
        pass
    except :
        import sys, traceback
        traceback.print_exc( file = sys.stderr )
        pass
    
    return None


#--------------------------------------------------------------------------------------
def add_options( the_option_parser, the_option_group ) :
    import pickup

    an_option_prefix = the_option_group.get_namespace()

    a_title = the_option_group.get_title()
    if a_title == None :
        a_title = an_option_prefix
        pass
    
    a_description = the_option_group.get_description()
    
    an_option_group = optparse.OptionGroup( the_option_parser, a_title, a_description )
    for an_option in the_option_group.option_list :
        if isinstance( an_option, PersistentOption ) :
            try:
                a_value = None
                an_expression = 'a_value = pickup.%s.%s' % ( an_option_prefix, an_option.dest )

                exec an_expression
                
                if a_value != None and a_value != 'None' :
                    an_option.default = a_value
                    pass
                
                pass
            except :
                import sys, traceback
                traceback.print_exc( file = sys.stderr )
                pass
            
            pass

        an_option_group.add_option( an_option )
        pass

    if len( the_option_group ) > 0 :
        the_option_parser.add_option_group( an_option_group )
        pass
    
    pass


#--------------------------------------------------------------------------------------
def template_add( the_option_parser, the_container ) :
    add_options( the_option_parser, the_container)
    pass


#------------------------------------------------------------------------------------------
def template_dump( the_identation_level, the_container, the_output ) :
    from balloon.preferences import dump_begin
    dump_begin( the_identation_level, the_container, the_output )

    from balloon.preferences import dump_end
    dump_end( the_identation_level, the_container, the_output )
    pass


#--------------------------------------------------------------------------------------
def correct_value( the_value ) :
    if the_value == "" or the_value == "''" :
        return None
    
    return the_value


#--------------------------------------------------------------------------------------
def get_input( the_args ) :
    if len( the_args ) > 0 :
        return correct_value( the_args[ 0 ] ), the_args[ 1 : ]

    try:
        a_values = raw_input()
    except:
        import sys, os; sys.exit( os.EX_UNAVAILABLE )
        pass

    a_values = a_values.split( ' ' )

    return correct_value( a_values[ 0 ] ), a_values[ 1 : ]


#--------------------------------------------------------------------------------------
def correct_values( the_values ) :
    a_values = []
    for a_value in the_values :
        if a_value == "" or a_value == "''" :
            a_values.append( None )
        else:
            a_values.append( a_value )
            pass
        pass
    
    return a_values


#--------------------------------------------------------------------------------------
def get_inputs( the_args ) :
    if len( the_args ) > 0 :
        return correct_values( the_args )

    try:
        a_values = [ raw_input() ]
    except:
        import sys, os; sys.exit( os.EX_UNAVAILABLE )
        pass
    
    return correct_values( a_values )


#--------------------------------------------------------------------------------------
